;; -*- mode: scheme -*-
;;
;; Gauche interactive session with readline support.
;;
;; By: YOKOTA Hiroshi

(use readline)
(use gauche.interactive)
(use srfi-1) ; for (append-map)

;; useful functions from gauche.interactive
(define /i info)
(define /d describe)
(define /a (lambda (obj)
	     (let1 key (if (string? obj)
			   (string->regexp obj)
			   obj)
		   (apropos key))))

(define (main args)
  (let ((readline-port (make-readline-input-port))
	(histfile (string-append (or (sys-getenv "HOME") ".")
				 "/.gosh_history")))
    (define (all-functions-list)
      (append-map (lambda (mod)
		    (hash-table-map (module-table mod)
				    (lambda (sym mod) (symbol->string sym))))
		  (all-modules)))

    (dynamic-wind
        (lambda ()
	  (rl-set-application-name!            "Gauche")
	  (rl-set-basic-word-break-characters! " \t\n()',\";")
	  (rl-set-basic-quote-characters!      "\"")
	  (rl-stifle-history!                  1000)
	  (rl-read-history!                    histfile)
	  (rl-set-completion-list-function!
	   (lambda () (all-functions-list))))
	(lambda ()
	  (read-eval-print-loop (lambda ()
				  (rl-set-prompt! "gosh> ")
				  (read readline-port)
				  )
				#f
				#f
				(lambda () #f)))
	(lambda ()
	  (rl-write-history histfile)))

    0))

;; end
