;;
;; readline module
;;
;; by: YOKOTA Hiroshi

(define-module readline
  (use gauche.uvector)
  (use gauche.vport)
  (export make-readline-input-port
	  rl-readline
	  rl-set-application-name!
	  rl-add-history! rl-read-history! rl-write-history rl-stifle-history!
	  rl-set-prompt!
	  rl-set-completion-list-function!
	  rl-set-basic-word-break-characters!
	  rl-set-basic-quote-characters!
	  ))
(select-module readline)

(dynamic-load "readline")

(define (make-readline-input-port)
  (make <buffered-input-port>
    :fill (lambda (vec)
	    (let ((ps1 rl-prompt)
		  (ps2 "... ")
		  (ps  #f)
		  (line #f))


	      (set! line (rl-readline ps1))
	      (rl-set-prompt! ps2)

		  (if (string? line)
		      (let* ((rlvec (string->u8vector line))
			     (len   (u8vector-length  rlvec)))

			(if (not (= (string-length line) 0))
			    (rl-add-history! line))

			(u8vector-copy! vec 0 rlvec)

			;; Add #\newline for line separator. Then add
			;; 1 to length and exits this function with
			;; length value.
			(u8vector-set! vec len (char->integer #\newline))
			(+ len 1))

		      0)))))

(define rl-prompt "readline> ")
(define (rl-set-prompt! str)
  (set! rl-prompt str))

(provide "readline")

;; end
